from .common import determine_wrapper as dwp
from .mpapi.qy import chat as qyChat

def send_some(msgType, mediaId, additionalDict={},
        targetIdList=[], partyIdList=[], tagIdList=[],
        agentId=None):
    return dwp(mpMsg.send_some, qyMsg.send_some,
        msgTyp, mediaId, additionalDict,
        targetIdList, partyIdList, tagIdList, agentId)

def create(chatId, name, ownerId, userIdList):
    return dwp(None, qyChat.create,
        chatId, name, ownerId, userIdList)

def get(chatId):
    return dwp(None, qyChat.get,
        chatId)

def update(chatId, opUserId, name=None, ownerId=None,
        addUserIdList=None, delUserIdList=None):
    return dwp(None, qyChat.update,
        chatId, opUserId, name, ownerId, addUserIdList, delUserIdList)

def quit(chatId, opUserId):
    return dwp(None, qyChat.quit,
        opUserId)

def clear_notify(ownerId, chatId=None, userId=None):
    return dwp(None, qyChat.clear_notify,
        ownerId, chatId, userId)

def send(msgType, mediaId, additionalDict={}, senderId=None,
        userId=None, chatId=None):
    return dwp(None, qyChat.send,
        msgTyp, mediaId, additionalDict, senderId, userId, chatId)

def set_mute(muteList=[], cancelList=[]):
    return dwp(None, qyChat.set_mute,
        muteList, cancelList)



from .common import determine_wrapper as dwp
from .mpapi.qy import oauth2 as qyOauth

def generate_code_url(redirectUri, state=None):
    return dwp(None, qyOauth.generate_code_url,
        redirectUri, state)

def get_user_info(code, accessToken=None):
    return dwp(None, qyOauth.get_user_info,
        code)

def user_id_open_id_switch(userId=None, openId=None, agentId=None):
    return dwp(None, qyOauth.user_id_open_id_switch,
        userId, openId, agentId)



import logging

from ..requests import requests
from itchatmp.config import COROUTINE
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry, encode_send_dict

logger = logging.getLogger('itchatmp')

def create_producer(serverUrl, access_token):
    def _create(menuDict, autoDecide=False, agentId=None):
        @access_token
        def __create(menuDict, agentId, accessToken):
            data = encode_send_dict(menuDict)
            if data is None: return ReturnValue({'errcode': -10001})
            url = '%s/cgi-bin/menu/create?access_token=%s' % \
                (serverUrl, accessToken)
            if agentId is not None: url += '&agentid=%s' % agentId
            r = requests.post(url, data=data)
            def _wrap_result(result):
                return ReturnValue(result.json())
            r._wrap_result = _wrap_result
            return r
        if autoDecide and not COROUTINE:
            currentMenu = get_producer(serverUrl, access_token)()
            if currentMenu.get('menu', {}) == menuDict:
                logger.debug('Menu already exists')
                return ReturnValue({'errcode': 0})
        return __create(menuDict, agentId)
    return _create

def get_producer(serverUrl, access_token):
    @access_token
    def _get(agentId=None, accessToken=None):
        url = '%s/cgi-bin/menu/get?access_token=%s' % (serverUrl, accessToken)
        if agentId is not None: url += '&agentid=%s' % agentId
        r = requests.get(url)
        def _wrap_result(result):
            result = result.json()
            if 'menu' in result: result['errcode'] = 0
            return ReturnValue(result)
        r._wrap_result = _wrap_result
        return r
    return _get

def delete_producer(serverUrl, access_token):
    @access_token
    def _delete(agentId=None, accessToken=None):
        url = '%s/cgi-bin/menu/delete?access_token=%s' % (serverUrl, accessToken)
        if agentId is not None: url += '&agentid=%s' % agentId
        r = requests.get(url)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    return _delete



from itchatmp.config import COMPANY_URL
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry
from ..base.common import (update_access_token_producer,
    access_token_producer, filter_request_producer)
from ..requests import requests

__all__ = ['update_access_token', 'access_token', 'get_server_ip', 'filter_request']

update_access_token = update_access_token_producer(
    COMPANY_URL + '/cgi-bin/gettoken?corpid=%s&corpsecret=%s',
    lambda x: x.config.copId)

access_token = access_token_producer(update_access_token)

def get_server_ip():
    @access_token
    def _get_server_ip(accessToken=None):
        url = '%s/cgi-bin/getcallbackip?access_token=%s' % \
            (COMPANY_URL, accessToken)
        r = requests.get(url).json()
        if 'ip_list' in r:
            r['errcode'] = 0
            for i, v in enumerate(r['ip_list']):
                r['ip_list'][i] = v[:v.rfind('/')]
        return ReturnValue(r)
    return _get_server_ip()

filter_request = filter_request_producer(get_server_ip)



#coding=utf8
import time, logging, mimetypes

import itchatmp
from itchatmp.content import (TEXT, MUSIC,
    IMAGE, VOICE, VIDEO, THUMB, NEWS, CARD)

itchatmp.update_config(itchatmp.WechatConfig(
    token='5tHMP2cZ6UOlscfqyUCuBr',
    appId = 'wx656b8e907615d60c',
    appSecret = 'd4624c36b6795d1d99dcf0547af5443d'))
itchatmp.update_config(filterRequest=True)

itchatmp.set_logging(loggingLevel=logging.DEBUG)

# userList = ['o5Bt8wSaeWlLjnGGwiwRJLS9_pvc', 'o5Bt8weoRUTxjpr_YDA1SX2JhMVA']
# r = itchatmp.menu.get(agentId=5)
# print(r)

@itchatmp.msg_register(itchatmp.content.TEXT)
def text_reply(msg):
    print(msg)
    return msg
    # return {'msgType': itchatmp.content.TEXT, 'content': msg['content']}
    # return '@img@' + msg['mediaId']

itchatmp.run(debug=True)



from .common import determine_wrapper
from .mpapi.mp import statistics as mpStat

'''
yiw"lpfuviwpyiw2jfu;viwp3k2d3j
'''

def user_summary(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_summary, None, *args)
def user_cumulate(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_cumulate, None, *args)
def article_summary(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.article_summary, None, *args)
def article_total(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.article_total, None, *args)
def user_read(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_read, None, *args)
def user_read_hour(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_read_hour, None, *args)
def user_share(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_share, None, *args)
def user_share_hour(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.user_share_hour, None, *args)
def upstream_msg(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg, None, *args)
def upstream_msg_hour(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_hour, None, *args)
def upstream_msg_week(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_week, None, *args)
def upstream_msg_month(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_month, None, *args)
def upstream_msg_dist(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_dist, None, *args)
def upstream_msg_dist_week(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_dist_week, None, *args)
def upstream_msg_dist_month(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.upstream_msg_dist_month, None, *args)
def interface_summary(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.interface_summary, None, *args)
def interface_summary_hour(startTime, timeSection=None):
    args = (startTime,) if timeSection is None else (startTime, timeSection)
    return determine_wrapper(mpStat.interface_summary_hour, None, *args)



import logging

class LogSystem(object):
    handlerList = []
    showOnCmd = True
    loggingLevel = logging.INFO
    loggingFile = None
    def __init__(self):
        self.cmdHandler = None
        for handler in logging.getLogger().handlers:
            if 'StreamHandler' in str(handler):
                self.cmdHandler = handler
        if self.cmdHandler is None:
            self.cmdHandler = logging.StreamHandler()
            logging.getLogger().addHandler(self.cmdHandler)
        self.logger = logging.getLogger('itchatmp')
        self.logger.addHandler(logging.NullHandler())
        self.logger.setLevel(self.loggingLevel)
        self.fileHandler = None
    def set_logging(self, showOnCmd=True, loggingFile=None,
            loggingLevel=logging.INFO):
        if showOnCmd != self.showOnCmd:
            if showOnCmd:
                logging.getLogger().addHandler(self.cmdHandler)
            else:
                logging.getLogger().removeHandler(self.cmdHandler)
            self.showOnCmd = showOnCmd
        if loggingFile != self.loggingFile:
            if self.loggingFile is not None: # clear old fileHandler
                self.logger.removeHandler(self.fileHandler)
                self.fileHandler.close()
            if loggingFile is not None: # add new fileHandler
                self.fileHandler = logging.FileHandler(loggingFile)
                self.logger.addHandler(self.fileHandler)
            self.loggingFile = loggingFile
        if loggingLevel != self.loggingLevel:
            self.logger.setLevel(loggingLevel)
            self.loggingLevel = loggingLevel

ls = LogSystem()
set_logging = ls.set_logging



from .server import WechatConfig, WechatServer
from .models.common import TestStorage
from .config import VERSION
from .controllers.envtest import env_test
from .controllers import (
    application, chat, common, menu,
    customerservice, messages, users, utils, statistics,
    templatemsgs, oauth2, wrapped)

from .log import set_logging

__version__ = VERSION

server = WechatServer(None, None, None)

update_config = server.update_config
run           = server.run
msg_register  = server.msg_register
send          = server.send
access_token  = server.access_token
clear_quota   = server.clear_quota
set_logging   = set_logging



from tornado.concurrent import Future

from itchatmp.server import WechatServer
from .mpapi.mp import common as mpCommon
from .mpapi.qy import common as qyCommon

server = WechatServer(None, None, None)

def determine_wrapper(mpFn=None, copFn=None, *args, **kwargs):
    if server.config.copId != '':
        if copFn is None:
            raise AttributeError('No such method for company platform')
        else:
            return copFn(*args, **kwargs)
    elif server.config.appId != '':
        if mpFn is None:
            raise AttributeError('No such method for massive platform')
        else:
            return mpFn(*args, **kwargs)
    else:
        raise AttributeError('You must specific appId or copId before use this method')

def update_access_token():
    return determine_wrapper(mpCommon.update_access_token,
        qyCommon.update_access_token)

def access_token(fn):
    return determine_wrapper(mpCommon.access_token, 
        qyCommon.access_token,
        fn)

def get_server_ip():
    return determine_wrapper(mpCommon.get_server_ip, 
        qyCommon.get_server_ip)

def filter_request(request):
    return determine_wrapper(mpCommon.filter_request, 
        qyCommon.filter_request,
        request)

def clear_quota():
    return determine_wrapper(mpCommon.clear_quota, None)

server.filter_request = filter_request
server.access_token = access_token
server.clear_quota = clear_quota



import hashlib

def oauth(timestamp, nonce, signature, token, echostr=None):
    ''' determine whether signature of request is right
     * both get and post functions need oauth
     * msgs we generate for sending don't match this fn
     * for Cop mp, we need echostr as well
    '''
    s = [timestamp, nonce, token]
    if echostr is not None: s.append(echostr)
    s.sort(); s = ''.join(s).encode('utf8')
    return hashlib.sha1(s).hexdigest() == signature



import re, logging

from tornado import gen

from itchatmp.config import COROUTINE
from itchatmp.content import (MUSIC,
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue
from itchatmp.views import reply_msg_format
from .customerservice import send as cssend
from .messages import preview, upload

logger = logging.getLogger('itchatmp')

def send(msg, toUserId, mediaId=None):
    ''' This is a method for sending messages to a specific user
     1. How this works?
        - if the user has contacted mp in 48 hours, use customerservice.send
        - if not, use messages.preview
        - MUSIC, which doesn't support preview will return 45015 (out of 48h)
     2. What should be passed into msg
        - it can be a string (for encoding other than ascii, unicode should be used)
          - it should start with things like @img@
          - then it should follow mediaId or fileDir (for text, it's content)
          - it supports: img, voc, vid, txt, nws, cad (msc is not supported)
        - it can be a dict (for encoding other than ascii, unicode should be used)
          - value of key "MsgType" should be the msgType
          - value of key "MediaId" should be mediaId (for text, value is content)
          - for MUSIC, there need to be three more keys:
            - musicurl, hqmusicurl, thumb_media_id
          - it supports all types: img, voc, vid, txt, nws, cad, msc
    '''
    msg = reply_msg_format(msg) # format string into dict
    # get msgType and mediaId
    if not ('MsgType' in msg and 'MediaId' in msg):
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'value of key "MsgType" should be the msgType and ' +
            'value of key "MediaId" should be mediaId (for text, value is content)'
            })
    else:
        msgType, content = msg['MsgType'], msg['MediaId']
    # filter unexpected messages
    if msgType not in (IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC):
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'send supports: IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC'})
    elif msgType == MUSIC and not ('musicurl' in msg and 'hqmusicurl' in msg):
        return ReturnValue({'errcode': -10003, 'errmsg': 
            'msg for type MUSIC should be: {"msgType": MUSIC, ' + 
            '"musicurl" :MUSICURL, "hqmusicurl" :HQMUSICURL, ' +
            '"thumb_media_id": MEDIA_ID}'})
    if COROUTINE:
        @gen.coroutine
        def _send():
            # deal with mediaId and fileDir
            if mediaId is not None and msgType != TEXT:
                c = mediaId
            elif 'FileDir' in msg:
                r = yield upload(msgType, msg['FileDir'])
                if not r: raise gen.Return(r)
                c = r['media_id']
            else:
                c = content
            r = yield cssend(msgType, c, additionalDict=msg, toUserId=toUserId)
            r['preview'] = False
            if not r:
                if r['errcode'] != 45015 or msgType == MUSIC: raise gen.Return(r)
                r = yield preview(msgType, content, additionalDict=msg, toUserId=toUserId)
                r['preview'] = True
            raise gen.Return(r)
        return _send()
    else:
        # deal with mediaId and fileDir
        if mediaId is not None and msgType != TEXT:
            content = mediaId
        elif 'FileDir' in msg:
            r = upload(msgType, msg['FileDir'])
            if not r: return r
            content = r['media_id']
        r = cssend(msgType, content, additionalDict=msg, toUserId=toUserId)
        r['preview'] = False
        if not r:
            if r['errcode'] != 45015 or msgType == MUSIC: return r
            r = preview(msgType, content, additionalDict=msg, toUserId=toUserId)
            r['preview'] = True
        return r



import logging, json
from datetime import datetime, timedelta

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue

DOC_DICT = {
    'getusersummary': ' get data of user following & unfollowing mp',
    'getusercumulate': ' get data of how many users following mp in all',
    'getarticlesummary': 'get data of daily news',
    'getarticletotal': 'get sum of all news',
    'getuserread': 'get amount of readers',
    'getuserreadhour': 'get amount of reading time',
    'getusershare': 'get amount of sharing',
    'getusersharehour': 'get amount of sharing time', 
    'getupstreammsg': 'get msg status',
    'getupstreammsghour': 'get msg status of detailed hour',
    'getupstreammsgweek': 'get msg status of weeks',
    'getupstreammsgmonth': 'get msg status of months',
    'getupstreammsgdist': 'get msg distribution',
    'getupstreammsgdistweek': 'get msg distribution of weeks',
    'getupstreammsgdistmonth': 'get msg distribution of months', 
    'getinterfacesummary': 'get interface summary',
    'getinterfacesummaryhour': 'get detailed hourly interface data',
    }

def format_time(startTime, timeSection, maxTimeSection=7):
    if not isinstance(timeSection, int) or not 1 <= timeSection <= maxTimeSection:
        return ReturnValue({'errcode': -10003, 'errmsg':
            'timeSection but be int between 1 to %s' % maxTimeSection}), None
    if hasattr(startTime, 'real'): # is number
        startTime = datetime.fromtimestamp(startTime)
    endTime = startTime + timedelta(timeSection - 1)
    return startTime.strftime("%Y-%m-%d"), endTime.strftime("%Y-%m-%d")

def fn_producer(fnName, maxTimeSection):
    def _fn_producer(startTime, timeSection=maxTimeSection):
        startTime, endTime = format_time(startTime, timeSection, maxTimeSection)
        if endTime is None: return startTime
        print(startTime, endTime)
        @access_token
        def __fn_producer(startTime, endTime, accessToken=None):
            data = {'begin_date': startTime, 'end_date': endTime}
            data = encode_send_dict(data)
            r = requests.post('%s/datacube/%s?access_token=%s' % 
                (SERVER_URL, fnName, accessToken), data=data)
            def _wrap_result(result):
                result = ReturnValue(result.json())
                if 'list' in result: result['errcode'] = 0
                return result
            r._wrap_result = _wrap_result
            return r
        return __fn_producer(startTime, endTime)
    _fn_producer.__doc__ = DOC_DICT[fnName] + \
        '\n * startTime can be timestamp or datetime.datetime' + \
        ('\n * timeSection must be a int from 1 to {t},' +
            '{t} for {t} days').format(t=maxTimeSection)
    return _fn_producer

user_summary            = fn_producer('getusersummary', 7)
user_cumulate           = fn_producer('getusercumulate', 7)
article_summary         = fn_producer('getarticlesummary', 1)
article_total           = fn_producer('getarticletotal', 1)
user_read               = fn_producer('getuserread', 3)
user_read_hour          = fn_producer('getuserreadhour', 1)
user_share              = fn_producer('getusershare', 7)
user_share_hour         = fn_producer('getusersharehour', 1)
upstream_msg            = fn_producer('getupstreammsg', 7)
upstream_msg_hour       = fn_producer('getupstreammsghour', 1)
upstream_msg_week       = fn_producer('getupstreammsgweek', 30)
upstream_msg_month      = fn_producer('getupstreammsgmonth', 30)
upstream_msg_dist       = fn_producer('getupstreammsgdist', 15)
upstream_msg_dist_week  = fn_producer('getupstreammsgdistweek', 30)
upstream_msg_dist_month = fn_producer('getupstreammsgdistmonth', 30)
interface_summary       = fn_producer('getinterfacesummary', 30)
interface_summary_hour  = fn_producer('getinterfacesummaryhour', 1)



import logging

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import COMPANY_URL
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def create(chatId, name, ownerId, userIdList, accessToken=None):
    if ownerId not in userIdList:
        return ReturnValue({'errcode': -10003, 'errmsg':
            'owner should be in userIdList'})
    data = {
        'chatid': chatId,
        'name': name,
        'owner': ownerId,
        'userlist': userIdList, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/create?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def get(chatId, accessToken=None):
    params = {
        'chatid': chatId,
        'access_token': accessToken, }
    r = requests.get('%s/cgi-bin/chat/get' % COMPANY_URL,
        params=params).json()
    return ReturnValue(r)

@access_token
def update(chatId, opUserId, name=None, ownerId=None,
        addUserIdList=None, delUserIdList=None, accessToken=None):
    data = {
        'chatid': chatId,
        'op_user': opUserId,
        'name': name,
        'owner': ownerId,
        'add_user_list': addUserIdList,
        'del_user_list': delUserIdList, }
    for k in list(data):
        if data[k] is None: del data[k]
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/update?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def quit(chatId, opUserId, accessToken=None):
    data = {
        'chatid': chatId,
        'op_user': opUserId, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/quit?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def clear_notify(ownerId, chatId=None, userId=None, accessToken=None):
    data = {
        'op_user': ownerId,
        'chat': {
            'type': 'group' if chatId else 'single',
            'id': chatId or userId, }}
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/clearnotify?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

@access_token
def send(msgType, mediaId, additionalDict={}, senderId=None,
        userId=None, chatId=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict: return msgDict
    msgDict['receiver'] = {
        'type': 'single' if userId else 'group',
        'id': userId or chatId, }
    msgDict['sender'] = senderId
    data = encode_send_dict(msgDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/send?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)

def __form_send_dict(msgType, mediaId, additionalDict):
    if not msgType in (TEXT, IMAGE, VOICE, VIDEO, FILE, NEWS):
        return ReturnValue({'errcode': 40004,})
    return {
        TEXT: {'text': {'content': mediaId}, 'msgtype': 'text'},
        IMAGE: {'image': {'media_id': mediaId}, 'msgtype': 'image'},
        VOICE: {'voice': {'media_id': mediaId}, 'msgtype': 'voice'},
        VIDEO: {'video':{'media_id': mediaId,
            'title': additionalDict.get('title', ''),
            'description': additionalDict.get('introduction', '')},
            'msgtype': 'video'},
        FILE: {'media_id': mediaId, 'msgtype': 'file'},
        NEWS: {'mpnews':{'media_id': mediaId}, 'msgtype': 'mpnews'},
        }[msgType]

@access_token
def set_mute(muteList=[], cancelList=[], accessToken=None):
    l = []
    for status, userIdList in enumerate((cancelList, muteList)):
        for userId in userIdList:
            l.append({'userid': userId,
                'status': status, })
    data = {'user_mute_list': l}
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/chat/setmute?access_token=%s' % 
        (COMPANY_URL, accessToken), data=data).json()
    return ReturnValue(r)



from .common import determine_wrapper as dwp
from .mpapi.mp import utils as mpUtils

def create_qrcode(sceneData, expire=2592000):
    return dwp(mpUtils.create_qrcode, None,
        sceneData, expire)

def download_qrcode(ticket):
    return dwp(mpUtils.download_qrcode, None,
        ticket)

def long_url_to_short(url):
    return dwp(mpUtils.long_url_to_short, None,
        url)



from .common import determine_wrapper as dwp
from .mpapi.mp import users as mpUsers
from .mpapi.qy import users as qyUsers

def create_tag(name, id=None):
    return dwp(mpUsers.create_tag, qyUsers.create_tag,
        name, id)

def get_tags():
    return dwp(mpUsers.get_tags, qyUsers.get_tags)

def update_tag(id, name):
    return dwp(mpUsers.update_tag, qyUsers.update_tag,
        id, name)

def delete_tag(id):
    return dwp(mpUsers.delete_tag, qyUsers.delete_tag,
        id)

def get_users_of_tag(id, nextOpenId=''):
    return dwp(mpUsers.get_users_of_tag, qyUsers.get_users_of_tag,
        id, nextOpenId)

def add_users_into_tag(id, userIdList=None, partyList=None):
    return dwp(mpUsers.add_users_into_tag, qyUsers.add_users_into_tag,
        id, userIdList, partyList)

def delete_users_of_tag(id, userIdList=None, partyList=None):
    return dwp(mpUsers.delete_users_of_tag, qyUsers.delete_users_of_tag,
        id, userIdList, partyList)

def get_tags_of_user(userId):
    return dwp(mpUsers.get_tags_of_user, None,
        userId)

def set_alias(userId, alias):
    return dwp(mpUsers.set_alias, None,
        userId, alias)

def get_user_info(userId):
    return dwp(mpUsers.get_user_info, qyUsers.get_user_info,
        userId)

def get_users(nextOpenId='', departmentId=None, fetchChild=False, status=4):
    return dwp(mpUsers.get_users, qyUsers.get_users,
        nextOpenId, departmentId, fetchChild, status)

def get_detailed_users(nextOpenId='', departmentId=None, fetchChild=False, status=4):
    return dwp(None, qyUsers.get_detailed_users,
        nextOpenId, departmentId, fetchChild, status)

def get_blacklist(beginOpenId=''):
    return dwp(mpUsers.create_tag, None,
        beginOpenId)

def add_users_into_blacklist(userId):
    return dwp(mpUsers.add_users_into_blacklist, None,
        userId)

def delete_users_of_blacklist(userId):
    return dwp(mpUsers.delete_users_of_blacklist, None,
        userId)

def authorize_user(userId):
    return dwp(None, qyUsers.authorize_user,
        userId)

def create_department(name, parentId=1, order=None, id=None):
    return dwp(None, qyUsers.create_department,
        name, parentId, order, id)

def update_department(id, name=None, parentId=None, order=None):
    return dwp(None, qyUsers.update_department,
        id, name, parentId, order)

def delete_department(id):
    return dwp(None, qyUsers.delete_department,
        id)

def get_departments(parentId):
    return dwp(None, qyUsers.get_departments,
        parentId)

def create_user(userId, name, departmentIdList,
        position=None, mobile=None, gender=None, email=None,
        weixinId=None, headImgId=None, extAttr=None):
    return dwp(None, qyUsers.create_user,
        userId, name, departmentIdList, position,
        mobile, gender, email, weixinId, headImgId, extAttr)

def update_user(userId, name=None, departmentIdList=None,
        position=None, mobile=None, gender=None, email=None,
        weixinId=None, headImgId=None, extAttr=None):
    return dwp(None, qyUsers.update_user,
        userId, name, departmentIdList, position,
        mobile, gender, email, weixinId, headImgId, extAttr)

def delete_users(userId):
    return dwp(None, qyUsers.delete_users,
        userId)

def upload_contract(csvMediaId, callbackUrl, method='sync'):
    return dwp(None, qyUsers.upload_contract,
        csvMediaId, callbackUrl, method)

def get_result(jobId):
    return dwp(None, qyUsers.get_result,
        jobId)



from requests import get

from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue
from itchatmp.server import WechatServer
from itchatmp.utils import retry
from ..base.common import (update_access_token_producer,
    access_token_producer, filter_request_producer)
from ..requests import requests

__all__ = ['update_access_token', 'access_token', 'get_server_ip', 'filter_request', 'clear_quota']

server = WechatServer(None, None, None)

update_access_token = update_access_token_producer(
    SERVER_URL + '/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s',
    lambda x: x.config.appId)

access_token = access_token_producer(update_access_token)

def get_server_ip_producer(forceSync=False):
    if COROUTINE and not forceSync:
        at = access_token
        requests_get = requests.get
    else:
        token_fn = update_access_token_producer(
            SERVER_URL + '/cgi-bin/token?grant_type=client_credential&appid=%s&secret=%s',
            lambda x: x.config.appId, forceSync=True)
        at = access_token_producer(token_fn, forceSync=True)
        requests_get = get
    @at
    def _get_server_ip(accessToken=None):
        url = '%s/cgi-bin/getcallbackip?access_token=%s' % \
            (SERVER_URL, accessToken)
        r = requests_get(url)
        def _wrap_result(result):
            result = ReturnValue(result.json())
            if 'ip_list' in result:
                result['errcode'] = 0
                for i, v in enumerate(result['ip_list']):
                    result['ip_list'][i] = v[:v.rfind('/')]
            return result
        r._wrap_result = _wrap_result
        return r
    return _get_server_ip

get_server_ip = get_server_ip_producer()

filter_request = filter_request_producer(get_server_ip_producer(True))

@access_token
def clear_quota(accessToken=None):
    data = {'appid': server.config.appId}
    r = requests.post('%s/cgi-bin/clear_quota?access_token=%s' %
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    return r



import time, json
import functools, logging, traceback

logger = logging.getLogger('itchatmp')

def retry(n=3, waitTime=3):
    def _retry(fn):
        @functools.wraps(fn)
        def __retry(*args, **kwargs):
            for i in range(n):
                try:
                    return fn(*args, **kwargs)
                except Exception as e:
                    logger.debug('%s failed. Count: %s. Info: %r' %
                        (fn.__name__, i + 1, e))
                    if i + 1 == n:
                        logger.debug('%s failed. Reach max retry' %
                            fn.__name__)
                    time.sleep(waitTime)
        return __retry
    return _retry

def encode_send_dict(d):
    try:
        return json.dumps(d).encode('utf8'). \
            decode('unicode-escape').encode('utf8')
    except (UnicodeDecodeError, UnicodeEncodeError):
        return



import logging, json, copy

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def set_industry(id1, id2, accessToken=None):
    ''' set industry for your massive platform '''
    data = {'industry_id1': str(id1), 'industry_id2': str(id2)}
    r = requests.post('%s/cgi-bin/template/api_set_industry?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(data))
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get_industry(accessToken=None):
    ''' get industry of your massive platform '''
    r = requests.get('%s/cgi-bin/template/get_industry?access_token=%s' % 
        (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'primary_industry' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def add_template(templateId, accessToken=None):
    ''' add template for your massive platform '''
    data = {'template_id_short': templateId}
    r = requests.post('%s/cgi-bin/template/api_add_template?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(data))
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def delete_templates(templateId, accessToken=None):
    ''' delete template of your massive platform '''
    data = {'template_id': templateId}
    r = requests.post('%s/cgi-bin/template/del_private_template?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(data))
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get_templates(accessToken=None):
    ''' get templates of your massive platform '''
    r = requests.get('%s/cgi-bin/template/get_all_private_template?access_token=%s' % 
        (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'template_list' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def send(templateId, msgDict, toUserId, accessToken=None):
    ''' send template to your massive platform users '''
    msgDict = copy.deepcopy(msgDict)
    msgDict['touser'], msgDict['template_id'] = toUserId, templateId
    data = encode_send_dict(msgDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/template/del_private_template?access_token=%s' % 
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r



import logging, json

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, MUSIC, TEXT, NEWS, CARD)
from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def get(accessToken=None):
    r = requests.post('%s/cgi-bin/customservice/getkflist?access_token=%s'
        % (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'kf_list' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def add(accountDict, autoDecide=True):
    @access_token
    def _add(accountDict, accessToken):
        data = encode_send_dict(accountDict)
        if data is None: return ReturnValue({'errcode': -10001})
        r = requests.post('%s/customservice/kfaccount/add?access_token=%s'
            % (SERVER_URL, accessToken), data=data)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    if autoDecide and not COROUTINE:
        currentList = get()
        for kf in currentList.get('kf_list', []):
            if kf['kf_account'] == accountDict.get('kf_account'):
                logger.debug('kf already exists')
                return ReturnValue({'errcode': 0})
    return _add(accountDict)

def update(accountDict, autoDecide=True):
    @access_token
    def _update(accountDict, accessToken):
        data = encode_send_dict(accountDict)
        if data is None: return ReturnValue({'errcode': -10001})
        r = requests.post('%s/customservice/kfaccount/add?access_token=%s'
            % (SERVER_URL, accessToken), data=data)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    if autoDecide and COROUTINE:
        currentList = get()
        for kf in currentList.get('kf_list', []):
            if kf['kf_account'] == accountDict.get('kf_account'):
                if kf['nickname'] == accountDict.get('nickname') and \
                        kf['password'] == accountDict.get('password'):
                    logger.debug('kf already have specific info')
                    break
        else:
            return ReturnValue({'errcode': 61452})
    return _update(accountDict)

def delete(accountDict, autoDecide=True):
    @access_token
    def _delete(accountDict, accessToken):
        data = encode_send_dict(accountDict)
        if data is None: return ReturnValue({'errcode': -10001})
        r = requests.post('%s/customservice/kfaccount/del?access_token=%s'
            % (SERVER_URL, accessToken), data=data)
        def _wrap_result(result):
            return ReturnValue(result.json())
        r._wrap_result = _wrap_result
        return r
    if autoDecide and COROUTINE:
        currentList = get()
        for kf in currentList.get('kf_list', []):
            if kf['kf_account'] == accountDict.get('kf_account'):
                return _delete(accountDict)
    else:
        return _delete(accountDict)
    return ReturnValue({'errcode': 61452})

@access_token
def set_head_image(openedFile, kfAccount, accessToken=None):
    r = requests.post('%s/customservice/kfaccount/uploadheadimg?'
        'access_token=%s&kf_account=%s' % 
        (SERVER_URL, accessToken, kfAccount),
        files={'file': openedFile})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def send(msgType, mediaId, additionalDict={}, toUserId='', accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict: return msgDict
    msgDict['touser'] = toUserId
    data = encode_send_dict(msgDict)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/message/custom/send?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

def __form_send_dict(msgType, mediaId, additionalDict):
    if not msgType in (IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD, MUSIC):
        return ReturnValue({'errcode': 40008,})
    elif msgType == MUSIC:
        if not ('musicurl' in additionalDict and 'hqmusicurl' in additionalDict
                and 'thumb_media_id' in additionalDict):
            return ReturnValue({'errcode': -10003, 'errmsg': 
                'additionalDict for type VIDEO should be: ' + 
                '{"musicurl" :MUSICURL, "hqmusicurl" :HQMUSICURL, ' +
                '"thumb_media_id": MEDIA_ID}'})
    elif msgType == NEWS:
        if 'articles' in additionalDict:
            msgType = 'mpnews'
    return {
        TEXT: {'text': {'content': mediaId}, 'msgtype': 'text'},
        IMAGE: {'image': {'media_id': mediaId}, 'msgtype': 'image'},
        VOICE: {'voice': {'media_id': mediaId}, 'msgtype': 'voice'},
        VIDEO: {'video': {'media_id': mediaId,
            'thumb_media_id': additionalDict.get('thumb_media_id', ''),
            'title': additionalDict.get('title', ''),
            'description': additionalDict.get('introduction', '')},
            'msgtype': 'video'},
        MUSIC: {'music': {
            'thumb_media_id': additionalDict.get('thumb_media_id', ''),
            'title': additionalDict.get('title', ''),
            'description': additionalDict.get('description', ''),
            'musicurl': additionalDict.get('musicurl', ''),
            'hqmusicurl': additionalDict.get('hqmusicurl', '')},
            'msgtype': 'music'},
        NEWS: {'news':{'articles': additionalDict.get('articles', [])},
            'msgtype': 'mpnews'},
        'mpnews': {'mpnews':{'media_id': mediaId}, 'msgtype': 'mpnews'},
        CARD: {'wxcard': {'card_id': mediaId}, 'msgtype': 'wxcard'},
        }[msgType]



from .common import determine_wrapper as dwp
from .mpapi.mp import customerservice as mpCs

def get():
    return dwp(mpCs.get, None)

def add(accountDict, autoDecide=True):
    return dwp(mpCs.add, None,
        accountDict, autoDecide)

def update(accountDict, autoDecide=True):
    return dwp(mpCs.update, None,
        accountDict, autoDecide)

def delete(accountDict, autoDecide=True):
    return dwp(mpCs.delete, None,
        accountDict, autoDecide)

def set_head_image(openedFile, kfAccount):
    return dwp(mpCs.set_head_image, None,
        openedFile, kfAccount)

def send(msgType, mediaId, additionalDict={}, toUserId=''):
    return dwp(mpCs.send, None,
        msgType, mediaId, additionalDict, toUserId)



from itchatmp.server import WechatServer
from .common import determine_wrapper as dwp
from .mpapi.mp import wrapped as mpWrapped

server = WechatServer(None, None, None)

def send(msg, toUserName, mediaId=None):
    return dwp(mpWrapped.send, None,
        msg, toUserName, mediaId)

server.send = send



from itchatmp.server import WechatServer
from .common import determine_wrapper
from .mpapi.mp import messages as mpMsg
from .mpapi.qy import messages as qyMsg

server = WechatServer(None, None, None)

def send_some(msgType, mediaId, additionalDict={},
        targetIdList=[], partyIdList=[], tagIdList=[],
        agentId=None):
    return determine_wrapper(mpMsg.send_some, qyMsg.send_some,
        msgTyp, mediaId, additionalDict,
        targetIdList, partyIdList, tagIdList, agentId)

def send_all(msgType, mediaId, additionalDict={}, tagId=None):
    return determine_wrapper(mpMsg.send_all, qyMsg.send_all,
        msgType, mediaId, additionalDict, tagId)

def preview(msgType, mediaId, additionalDict={}, toUserId=None, toWxAccount=None):
    return determine_wrapper(mpMsg.preview, None,
        msgType, mediaId, additionalDict, toUserId, toWxAccount)

def delete(msgId):
    return determine_wrapper(mpMsg.delete, None,
        msgId)

def get(msgId):
    return determine_wrapper(mpMsg.get, None,
        msgId)

def upload(fileType, fileDir, additionalDict={}, permanent=False):
    return determine_wrapper(mpMsg.upload, qyMsg.upload,
        fileType, fileDir, additionalDict, permanent)

def download(mediaId):
    return determine_wrapper(mpMsg.download, qyMsg.download,
        mediaId)

def get_material(mediaId):
    return determine_wrapper(mpMsg.get_material, qyMsg.get_material,
        mediaId)

def delete_material(mediaId):
    return determine_wrapper(mpMsg.delete_material, qyMsg.delete_material,
        mediaId)

def get_materialcount():
    return determine_wrapper(mpMsg.get_materialcount, qyMsg.get_materialcount)

def batchget_material(fileType, offset=0, count=20):
    return determine_wrapper(mpMsg.batchget_material, qyMsg.batchget_material,
        fileType, offset, count)

def create_news(newsDict, permanent=False):
    return determine_wrapper(mpMsg.create_news, qyMsg.create_news,
        newsDict, permanent)

def update_news(mediaId, newsDict, index=0):
    return determine_wrapper(mpMsg.update_news, qyMsg.update_news,
        mediaId, newsDict, index)

def get_image_url(openedFile):
    return determine_wrapper(mpMsg.get_image_url, qyMsg.get_image_url,
        openedFile)

server.upload = upload



from itchatmp.config import COMPANY_URL
from itchatmp.returnvalues import ReturnValue
from itchatmp.utils import retry, encode_send_dict
from ..requests import requests
from .common import access_token

@access_token
def get(agentId, accessToken=None):
    params = {
        'access_token': accessToken,
        'agentid': agentId, }
    r = requests.get('%s/cgi-bin/agent/get' % COMPANY_URL,
        params=params).json()
    return ReturnValue(r)

@access_token
def set(agentId, **kwargs):
    kwargs['agentid'] = agentId
    data = encode_send_dict(kwargs)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/agent/set?access_token=%s' %
        (COMPANY_URL, kwargs['accessToken']), data=data).json()
    return ReturnValue(r)

@access_token
def list(accessToken=None):
    r = requests.get('%s/cgi-bin/agent/list?access_token=%s' % \
        (COMPANY_URL, accessToken)).json()
    return ReturnValue(r)



import logging, json

from ..requests import requests
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL, COROUTINE
from itchatmp.returnvalues import ReturnValue
from .common import access_token
from ..base.menu import create_producer, get_producer, delete_producer

logger = logging.getLogger('itchatmp')

create = create_producer(SERVER_URL, access_token)

get = get_producer(SERVER_URL, access_token)

delete = delete_producer(SERVER_URL, access_token)

def addconditional(menuDict, autoDecide=False):
    @access_token
    def _add(menuDict, accessToken):
        data = encode_send_dict(menuDict)
        if data is None: return ReturnValue({'errcode': -10001})
        r = requests.post('%s/cgi-bin/menu/addconditional?access_token=%s' % (
            SERVER_URL, accessToken), data=data)
        def _wrap_result(result):
            result = ReturnValue(result.json())
            if 'menuid' in result: result['errcode'] = 0
            return result
        r._wrap_result = _wrap_result
        return r
    if not ('button' in menuDict and 'matchrule' in menuDict):
        return ReturnValue({'errcode': 40035})
    if autoDecide and not COROUTINE:
        currentMenu = get()
        for cm in currentMenu.get('conditionalmenu', []):
            if cm.get('button', []) == currentMenu['button'] and \
                    cm.get('matchrule', {}) == currentMenu['matchrule']:
                logger.debug('exists conditional menu with same content')
                return ReturnValue({'errcode': 0})
    return _add(menuDict)
    
@access_token
def delconditional(menuId, accessToken=None):
    r = requests.post('%s/cgi-bin/menu/delconditional?access_token=%s' % (
        SERVER_URL, accessToken), data={'menuid': menuId})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def trymatch(userId, accessToken=None):
    ''' get menu of specific user
     * userId can be wechat account or openId
    '''
    r = requests.post('%s/cgi-bin/menu/trymatch?access_token=%s' % (
        SERVER_URL, accessToken), data={'user_id': userId})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get_current_selfmenu_info(accessToken=None):
    r = requests.get(
        '%s/cgi-bin/menu/get_current_selfmenu_info?access_token=%s'
        % (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'selfmenu_info' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r



from .common import determine_wrapper as dwp
from .mpapi.mp import templatemsgs as mpTMsg

def set_industry(id1, id2):
    return dwp(mpTMsg.set_industry, None,
        id1, id2)

def get_industry():
    return dwp(mpTMsg.get_industry, None)

def add_template(templateId):
    return dwp(mpTMsg.add_template, None,
        templateId)

def delete_templates(templateId):
    return dwp(mpTMsg.delete_templates, None,
        templateId)

def get_templates():
    return dwp(mpTMsg.get_templates, None)

def send(templateId, msgDict, toUserId):
    return dwp(mpTMsg.send, None,
        templateId, msgDict, toUserId)



from itchatmp.config import COMPANY_URL
from ..base.menu import create_producer, get_producer, delete_producer
from .common import access_token

create = create_producer(COMPANY_URL, access_token)

get = get_producer(COMPANY_URL, access_token)

delete = delete_producer(COMPANY_URL, access_token)



NORMAL, COMPATIBLE, SAFE = 0, 1, 2

TEXT = 'text'
IMAGE = 'image'
VOICE = 'voice'
VIDEO = 'video'
SHORT_VIDEO = 'shortvideo'
LOCATION = 'location'
LINK = 'link'
MUSIC = 'music'
NEWS = 'news'
CARD = 'card'
THUMB = 'thumb'
FILE = 'file'
TRANSFER = 'transfer_customer_service'
class _EVENT(object):
    SUBSCRIBE = 'subscribe'
    SCAN = 'SCAN'
    LOCATION = 'LOCATION'
    CLICK = 'CLICK'
    VIEW = 'VIEW'
    def __eq__(self, other):
        return 'event' == other
    def __hash__(self):
        return hash('event')
EVENT = _EVENT()
ENCRYPT = 'encrypt'

INCOME_MSG = [TEXT, IMAGE, VOICE, VIDEO, SHORT_VIDEO, LOCATION,
    LINK, EVENT]

OUTCOME_MSG = [TEXT, IMAGE, VOICE, VIDEO, MUSIC, NEWS, CARD]



''' This package is for mass texting in wechat mp
 1. What can we send?
    - IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD

 2. How to send them?
    - we use send_some / send_all method
        `send_some(targetIdList, msgType, mediaId, additionalDict)`
    - for msg like text and card, just pass content as msgId
    - for files like image, voice, video, we need to upload them first
        `upload(fileType, openedFile, additionalDict, permanent)`
    - for news, you need to form them first and upload to get msgId
        `create_news(newsDict, permanent)`
      for images used in news, you need to turn them into url first
        `get_image_url(openedFile)`
    - SPECIAL WARNING: video is a little bit **STRANGE**
      when uploading or sending, you need to pass additionalDict to method
        `{"title" :VIDEO_TITLE, "introduction" :INTRODUCTION}`

 3. I alse listed API list for you:
   - SENDING
     send_some
     send_all
     preview
   - MSG MANAGING
     delete
     get
   - TEMP MATERIAL MANAGING
     upload
     download
   - PERMENENT MATERIAL MANAGING
     get_material
     delete_material
     get_materialcount
     batchget_material
   - FORM NEWS
     create_news
     update_news
     get_image_url
'''
import logging, json, os, mimetypes

from ..requests import requests
from .common import access_token
from itchatmp.utils import retry, encode_send_dict
from itchatmp.config import SERVER_URL
from itchatmp.content import (
    IMAGE, VOICE, VIDEO, THUMB, TEXT, NEWS, CARD)
from itchatmp.returnvalues import ReturnValue

logger = logging.getLogger('itchatmp')

@access_token
def send_some(msgType, mediaId, additionalDict={},
        targetIdList=[], partyIdList=[], tagIdList=[],
        agentId=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict: return msgDict
    if not isinstance(targetIdList, list) or len(targetIdList) < 2:
        return ReturnValue({'errcode': 40130})
    msgDict['touser'] = targetIdList
    r = requests.post('%s/cgi-bin/message/mass/send?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(msgDict))
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def send_all(msgType, mediaId, additionalDict={}, tagId=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict: return msgDict
    if tagId is None: 
        msgDict['filter'] = {'is_to_all': True, 'tag_id': 0}
    else:
        msgDict['filter'] = {'is_to_all': False, 'tag_id': tagId}
    r = requests.post('%s/cgi-bin/message/mass/sendall?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(msgDict))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'media_id' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def preview(msgType, mediaId, additionalDict={},
        toUserId=None, toWxAccount=None, accessToken=None):
    msgDict = __form_send_dict(msgType, mediaId, additionalDict)
    if not msgDict: return msgDict
    if (toUserId or toWxAccount) is None:
        return ReturnValue({'errcode': -10003})
    else:
        if toUserId is not None: msgDict['touser'] = toUserId
        if toWxAccount is not None: msgDict['towxname'] = toWxAccount
    r = requests.post('%s/cgi-bin/message/mass/preview?access_token=%s' % 
        (SERVER_URL, accessToken), data=encode_send_dict(msgDict))
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def form_video_id(mediaId, additionalDict, accessToken=None):
    ''' in theory video needs another method to get media_id for sending '''
    additionalDict['media_id'] = mediaId
    additionalDict['description'] = additionalDict['introduction']
    # requests.packages.urllib3.disable_warnings()
    url = 'https://file.api.weixin.qq.com/cgi-bin/media/uploadvideo' \
        '?access_token=%s' % accessToken
    r = requests.post(url, data=encode_send_dict(additionalDict))
        # verify=False).json()
    # I don't know why this is a fake ssl
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'media_id' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

def __form_send_dict(msgType, mediaId, additionalDict):
    if not msgType in (IMAGE, VOICE, VIDEO, TEXT, NEWS, CARD):
        return ReturnValue({'errcode': 40004,})
    elif msgType == VIDEO:
        mediaId = form_video_id(mediaId, additionalDict)['media_id']
        if not mediaId: return mediaId
    return {
        NEWS: {'mpnews':{'media_id': mediaId}, 'msgtype': 'mpnews'},
        TEXT: {'text': {'content': mediaId}, 'msgtype': 'text'},
        VOICE: {'voice': {'media_id': mediaId}, 'msgtype': 'voice'},
        IMAGE: {'image': {'media_id': mediaId}, 'msgtype': 'image'},
        VIDEO: {'mpvideo':{'media_id': mediaId,
            'title': additionalDict.get('title', ''),
            'description': additionalDict.get('introduction', '')},
            'msgtype': 'mpvideo'},
        CARD: {'wxcard': {'card_id': mediaId}, 'msgtype': 'wxcard'},
        }[msgType]

@access_token
def delete(msgId, accessToken=None):
    r = requests.post('%s/cgi-bin/message/mass/delete?access_token=%s' % 
        (SERVER_URL, accessToken), data={'msg_id': msgId})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get(msgId, accessToken=None):
    r = requests.post('%s/cgi-bin/message/mass/get?access_token=%s' % 
        (SERVER_URL, accessToken), data={'msg_id': int(msgId)})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def upload(fileType, fileDir, additionalDict={}, permanent=False, accessToken=None):
    if not fileType in (IMAGE, VOICE, VIDEO, THUMB):
        return ReturnValue({'errcode': 40004,})
    try:
        with open(fileDir, 'rb') as f: file = f.read()
    except:
        return ReturnValue({'errcode': -10004,})
    fileName = 'file' + os.path.splitext(fileDir)[1].decode('utf8', 'replace')
    fileMime = mimetypes.guess_type(fileName)[0] or 'application/octet-stream'
    if fileType == VIDEO and not ('title' in additionalDict
            and 'introduction' in additionalDict):
        return ReturnValue({'errcode': -10001, 'errmsg': 
            'additionalDict for type VIDEO should be: ' + 
            '{"title" :VIDEO_TITLE, "introduction" :INTRODUCTION}'})
    if permanent:
        url = '%s/cgi-bin/material/add_material?access_token=%s&type=%s'
    else:
        url = '%s/cgi-bin/media/upload?access_token=%s&type=%s' 
    files = {'media': (fileName, file, fileMime), }
    if fileType == VIDEO:
        files['description'] = (None,
            encode_send_dict(additionalDict), 'application/json'),
    r = requests.post(url % (SERVER_URL, accessToken, fileType),
        files=files)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'media_id' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def download(mediaId, accessToken=None):
    r = requests.get('%s/cgi-bin/media/get?access_token=%s&media_id=%s' % 
        (SERVER_URL, accessToken, mediaId), stream=True)
    def _wrap_result(result):
        if 'application/json' in result.headers['Content-Type']:
            return ReturnValue(result.json())
        else:
            tempStorage = io.BytesIO()
            for block in result.iter_content(1024):
                tempStorage.write(block)
            return ReturnValue({'file': tempStorage, 'errcode': 0})
    r._wrap_result = _wrap_result
    return r

@access_token
def get_material(mediaId, accessToken=None):
    data = {'media_id': mediaId}
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/material/get_material?access_token=%s' % 
        (SERVER_URL, accessToken), data=data, stream=True)
    def _wrap_result(result):
        if 'application/json' in result.headers['Content-Type']:
            j = result.json()
            if 'news_item' in j or 'down_url' in j: j['errcode'] = 0
            return ReturnValue(j)
        else:
            tempStorage = io.BytesIO()
            for block in result.iter_content(1024):
                tempStorage.write(block)
            return ReturnValue({'file': tempStorage, 'errcode': 0})
    r._wrap_result = _wrap_result
    return r

@access_token
def delete_material(mediaId, accessToken=None):
    r = requests.post('%s/cgi-bin/material/del_material?access_token=%s' % 
        (SERVER_URL, accessToken), data={'msg_id': mediaId})
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get_materialcount(accessToken=None):
    r = requests.get('%s/cgi-bin/material/get_materialcount?access_token=%s'
        % (SERVER_URL, accessToken))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'voice_count' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def batchget_material(fileType, offset=0, count=20, accessToken=None):
    if not fileType in (IMAGE, VOICE, VIDEO, THUMB):
        return ReturnValue({'errcode': 40004,})
    if 20 < count: count = 20
    data = {'type': fileType,
        'offset': offset,
        'count': count, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/material/batchget_material?access_token=%s'
        % (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'total_count' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def create_news(newsDict, permanent=False, accessToken=None):
    if permanent:
        url = '%s/cgi-bin/material/add_news?access_token=%s'
    else:
        url = '%s/cgi-bin/media/uploadnews?access_token=%s'
    r = requests.post(url % (SERVER_URL, accessToken),
        data=encode_send_dict(newsDict))
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'media_id' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r

@access_token
def update_news(mediaId, newsDict, index=0, accessToken=None):
    data = {
        'media_id': mediaId,
        'index': index,
        'articles': newsDict, }
    data = encode_send_dict(data)
    if data is None: return ReturnValue({'errcode': -10001})
    r = requests.post('%s/cgi-bin/material/update_news?access_token=%s' %
        (SERVER_URL, accessToken), data=data)
    def _wrap_result(result):
        return ReturnValue(result.json())
    r._wrap_result = _wrap_result
    return r

@access_token
def get_image_url(openedFile, accessToken=None):
    r = requests.post('%s/cgi-bin/media/uploadimg?access_token=%s' % 
        (SERVER_URL, accessToken), files={'file': openedFile})
    def _wrap_result(result):
        result = ReturnValue(result.json())
        if 'url' in result: result['errcode'] = 0
        return result
    r._wrap_result = _wrap_result
    return r



from .common import determine_wrapper as dwp
from .mpapi.qy import application as qyApp

def get(agentId):
    return dwp(None, qyApp.get,
        agentId)

def set(agentId, **kwargs):
    return dwp(None, qyApp.set,
        agentId, **kwargs)

def list():
    return dwp(None, qyApp.list)



from .common import determine_wrapper as dwp
from .mpapi.mp import menu as mpMenu
from .mpapi.qy import menu as qyMenu

def create(menuDict, autoDecide=False, agentId=None):
    return dwp(mpMenu.create, qyMenu.create,
        menuDict, autoDecide, agentId)

def get(agentId=None):
    return dwp(mpMenu.get, qyMenu.get,
        agentId)

def delete(agentId=None):
    return dwp(mpMenu.delete, qyMenu.delete,
        agentId)

def addconditional(menuDict, autoDecide=False):
    return dwp(mpMenu.addconditional, None,
        menuDict, autoDecide)
    
def delconditional(menuId):
    return dwp(mpMenu.delconditional, None,
        menuId)

def trymatch(userId):
    return dwp(mpMenu.trymatch, None,
        userId)

def get_current_selfmenu_info():
    return dwp(mpMenu.get_current_selfmenu_info, None)



from itchatmp.config import COROUTINE

# if you have itchatmphttp installed
# we will use coroutine requests instead
if COROUTINE:
    try:
        from itchatmphttp import requests
    except ImportError:
        raise ImportError('You must installed itchatmphttp to use coroutine features')
else:
    import requests
    requests.packages.urllib3.disable_warnings()
    requests = requests.session()
    requests.verify = False



import functools, logging, time, threading
from datetime import datetime, timedelta

from requests.models import Response
from requests.sessions import session
from tornado import gen

from ..requests import requests
from itchatmp.config import COROUTINE
from itchatmp.server import WechatServer
from itchatmp.utils import retry
from itchatmp.returnvalues import ReturnValue

__all__ = ['update_access_token_producer', 'access_token', 'filter_request']

logger = logging.getLogger('itchatmp')

__server = WechatServer(None, None, None)
__AUTO_MAINTAIN = False
__serverList = None
__session = session()
__session.verify = False
get = __session.get

def auto_maintain_thread(firstCallResult=None, tokenUrl=None, appIdFn=None):
    r = firstCallResult or update_access_token_producer(tokenUrl, appIdFn, True)()
    if not r:
        __server.ioLoop.call_later(
            (datetime.replace(datetime.now() + timedelta(days=1), 
            hour=0, minute=5, second=0) - datetime.now()).seconds,
            maintain_access_token,
            None, tokenUrl, appIdFn)
    else:
        __server.ioLoop.call_later(r['expires_in'] - 30,
            maintain_access_token,
            None, tokenUrl, appIdFn)

def maintain_access_token(firstCallResult=None, tokenUrl=None, appIdFn=None):
    t = threading.Thread(target=auto_maintain_thread,
        args=(firstCallResult, tokenUrl, appIdFn))
    t.setDaemon(True); t.start()

def update_access_token_producer(tokenUrl, appIdFn, forceSync=False):
    ''' function to update access token
     * auto-maintain begins when this function is called for the first time
       - of course, if isWsgi or haven't started, auto-maintain will not start
     * If auto-maintain failed, it will restart tomorrow 0004h:30
     * will return ReturnValue object (equals to True) if success
     * will return ReturnValue object (equals to False) if fail
     * ReturnValue object contains why update failed
    '''
    if COROUTINE and not forceSync:
        @gen.coroutine
        def _update_access_token():
            global __AUTO_MAINTAIN
            url = tokenUrl % (appIdFn(__server), __server.config.appSecret)
            r = (yield requests.get(url)).json()
            if 'access_token' in r:
                __server.atStorage.store_access_token(
                    r['access_token'], int(time.time()) + r['expires_in'])
                r['errcode'] = 0
                r = ReturnValue(r)
            else:
                r = ReturnValue(r)
                logger.debug('Failed to get token: %s' %r)
            if not __AUTO_MAINTAIN and not __server.isWsgi:
                __AUTO_MAINTAIN = True
                maintain_access_token(r, tokenUrl, appIdFn)
            raise gen.Return(r)
        return _update_access_token
    else:
        def _update_access_token():
            global __AUTO_MAINTAIN
            url = tokenUrl % (appIdFn(__server), __server.config.appSecret)
            r = get(url).json()
            if 'access_token' in r:
                __server.atStorage.store_access_token(
                    r['access_token'], int(time.time()) + r['expires_in'])
                r['errcode'] = 0
                r = ReturnValue(r)
            else:
                r = ReturnValue(r)
                logger.debug('Failed to get token: %s' %r)
            if not __AUTO_MAINTAIN and not __server.isWsgi:
                __AUTO_MAINTAIN = True
                maintain_access_token(r, tokenUrl, appIdFn)
            return r
        return _update_access_token

def access_token_producer(tokenFn, forceSync=False):
    ''' wrapper for functions need accessToken
        accessToken should be a key argument of the wrapped fn
        ..code:: python
             @access_token
             def fn(a, b, c, accessToken=False):
                 pass
        if accessToken is not successfully fetched, wrapped fn will return why
        There's a very tricky thing about this decorator:
            You need to return a requests.models.Response object
            And set _wrap_result of the object to format result
    '''
    if COROUTINE and not forceSync:
        def _access_token(fn):
            @gen.coroutine
            @functools.wraps(fn)
            def _access_token(*args, **kwargs):
                accessToken, expireTime = __server.atStorage.get_access_token()
                if expireTime < time.time():
                    updateResult = yield tokenFn()
                    if not updateResult: raise gen.Return(updateResult)
                    accessToken, expireTime = __server.atStorage.get_access_token()
                kwargs['accessToken'] = accessToken
                future = fn(*args, **kwargs)
                r = yield future
                if isinstance(r, Response) and r.json().get('errcode') == 40014: # token timeout
                    updateResult = yield tokenFn()
                    if not updateResult: raise gen.Return(updateResult)
                    accessToken, expireTime = __server.atStorage.get_access_token()
                    kwargs['accessToken'] = accessToken
                    future = fn(*args, **kwargs)
                    r = yield future
                wrap_fn = getattr(future, '_wrap_result', None)
                if wrap_fn is not None: r = wrap_fn(r)
                raise gen.Return(r)
            return _access_token
        return _access_token
    else:
        def _access_token(fn):
            @functools.wraps(fn)
            def _access_token(*args, **kwargs):
                accessToken, expireTime = __server.atStorage.get_access_token()
                if expireTime < time.time():
                    updateResult = tokenFn()
                    if not updateResult: return updateResult
                    accessToken, expireTime = __server.atStorage.get_access_token()
                kwargs['accessToken'] = accessToken
                r = fn(*args, **kwargs)
                if isinstance(r, Response) and r.json().get('errcode') == 40014: # token timeout
                    updateResult = tokenFn()
                    if not updateResult: return updateResult
                    accessToken, expireTime = __server.atStorage.get_access_token()
                    kwargs['accessToken'] = accessToken
                    r = fn(*args, **kwargs)
                wrap_fn = getattr(r, '_wrap_result', None)
                if wrap_fn is not None: r = wrap_fn(r)
                return r
            return _access_token
        return _access_token

def set_server_list(get_server_ip):
    global __serverList
    __serverList = []
    serverList, fetchTime = __server.atStorage.get_server_list()
    if fetchTime < time.mktime(datetime.replace(datetime.now(),
        hour=0, minute=0, second=0).timetuple()):
        r = get_server_ip()
        if not r: logger.debug(r)
        __serverList = r.get('ip_list', [])
        __server.atStorage.store_server_list(__serverList, time.time())
    else:
        __serverList = serverList

def filter_request_producer(get_server_ip):
    def filter_request(request):
        global __serverList
        if __serverList is None:
            t = threading.Thread(target=set_server_list, args=(get_server_ip,))
            t.setDaemon = True
            t.start()
            def clear_server_list():
                __serverList = None
            __server.ioLoop.call_later(
                (datetime.replace(datetime.now() + timedelta(days=1), 
                hour=0, minute=5, second=0) - datetime.now()).seconds,
                lambda: clear_server_list())
        if not __serverList:
            logger.debug('Server list is loading, so ignore verifying once.')
            return True
        return request.remote_ip in __serverList
    return filter_request
